@@ -46,7 +46,7 @@ namespace mediascanner {
 
 // Increment this whenever changing db schema.
 // It will cause dbstore to rebuild its tables.
-static const int schemaVersion = 8;
+static const int schemaVersion = 9;
 
 struct MediaStorePrivate {
     sqlite3 *db;
@@ -70,6 +70,7 @@ struct MediaStorePrivate {
     std::vector<std::string> listArtists(const Filter &filter) const;
     std::vector<std::string> listAlbumArtists(const Filter &filter) const;
     std::vector<std::string> listGenres(const Filter &filter) const;
+    bool hasMedia(MediaType type) const;
 
     size_t size() const;
     void pruneDeleted();
@@ -292,7 +293,8 @@ void createTables(sqlite3 *db) {
 CREATE TABLE schemaVersion (version INTEGER);
 
 CREATE TABLE media (
-    filename TEXT PRIMARY KEY NOT NULL CHECK (filename LIKE '/%'),
+    id INTEGER PRIMARY KEY,
+    filename TEXT UNIQUE NOT NULL CHECK (filename LIKE '/%'),
     content_type TEXT,
     etag TEXT,
     title TEXT,
@@ -309,16 +311,19 @@ CREATE TABLE media (
     latitude DOUBLE,
     longitude DOUBLE,
     has_thumbnail INTEGER CHECK (has_thumbnail IN (0, 1)),
+    mtime INTEGER,
     type INTEGER CHECK (type IN (1, 2, 3)) -- MediaType enum
 );
 
 CREATE INDEX media_type_idx ON media(type);
-CREATE INDEX media_song_info_idx ON media(type, album_artist, album, disc_number, track_number, title) WHERE type = 0;
-CREATE INDEX media_artist_idx ON media(type, artist) WHERE type = 0;
-CREATE INDEX media_genre_idx ON media(type, genre) WHERE type = 0;
+CREATE INDEX media_song_info_idx ON media(type, album_artist, album, disc_number, track_number, title) WHERE type = 1;
+CREATE INDEX media_artist_idx ON media(type, artist) WHERE type = 1;
+CREATE INDEX media_genre_idx ON media(type, genre) WHERE type = 1;
+CREATE INDEX media_mtime_idx ON media(type, mtime);
 
 CREATE TABLE media_attic (
-    filename TEXT PRIMARY KEY NOT NULL,
+    id INTEGER PRIMARY KEY,
+    filename TEXT UNIQUE NOT NULL,
     content_type TEXT,
     etag TEXT,
     title TEXT,
@@ -335,26 +340,27 @@ CREATE TABLE media_attic (
     latitude DOUBLE,
     longitude DOUBLE,
     has_thumbnail INTEGER,
+    mtime INTEGER,
     type INTEGER   -- 0=Audio, 1=Video
 );
 
-CREATE VIRTUAL TABLE media_fts 
+CREATE VIRTUAL TABLE media_fts
 USING fts4(content='media', title, artist, album, tokenize=mozporter);
 
 CREATE TRIGGER media_bu BEFORE UPDATE ON media BEGIN
-  DELETE FROM media_fts WHERE docid=old.rowid;
+  DELETE FROM media_fts WHERE docid=old.id;
 END;
 
 CREATE TRIGGER media_au AFTER UPDATE ON media BEGIN
-  INSERT INTO media_fts(docid, title, artist, album) VALUES (new.rowid, new.title, new.artist, new.album);
+  INSERT INTO media_fts(docid, title, artist, album) VALUES (new.id, new.title, new.artist, new.album);
 END;
 
 CREATE TRIGGER media_bd BEFORE DELETE ON media BEGIN
-  DELETE FROM media_fts WHERE docid=old.rowid;
+  DELETE FROM media_fts WHERE docid=old.id;
 END;
 
 CREATE TRIGGER media_ai AFTER INSERT ON media BEGIN
-  INSERT INTO media_fts(docid, title, artist, album) VALUES (new.rowid, new.title, new.artist, new.album);
+  INSERT INTO media_fts(docid, title, artist, album) VALUES (new.id, new.title, new.artist, new.album);
 END;
 
 CREATE TABLE broken_files (
@@ -431,7 +437,7 @@ size_t MediaStorePrivate::size() const {
 }
 
 void MediaStorePrivate::insert(const MediaFile &m) const {
-    Statement query(db, "INSERT OR REPLACE INTO media (filename, content_type, etag, title, date, artist, album, album_artist, genre, disc_number, track_number, duration, width, height, latitude, longitude, has_thumbnail, type)  VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
+    Statement query(db, "INSERT OR REPLACE INTO media (filename, content_type, etag, title, date, artist, album, album_artist, genre, disc_number, track_number, duration, width, height, latitude, longitude, has_thumbnail, mtime, type)  VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
     query.bind(1, m.getFileName());
     query.bind(2, m.getContentType());
     query.bind(3, m.getETag());
@@ -449,7 +455,8 @@ void MediaStorePrivate::insert(const MediaFile &m) const {
     query.bind(15, m.getLatitude());
     query.bind(16, m.getLongitude());
     query.bind(17, (int)m.getHasThumbnail());
-    query.bind(18, (int)m.getType());
+    query.bind(18, (int64_t)m.getModificationTime());
+    query.bind(19, (int)m.getType());
     query.step();
 
     const char *typestr = m.getType() == AudioMedia ? "song" : "video";
@@ -509,7 +516,8 @@ static MediaFile make_media(Statement &query) {
         .setLatitude(query.getDouble(14))
         .setLongitude(query.getDouble(15))
         .setHasThumbnail(query.getInt(16))
-        .setType((MediaType)query.getInt(17));
+        .setModificationTime(query.getInt64(17))
+        .setType((MediaType)query.getInt(18));
 }
 
 static vector<MediaFile> collect_media(Statement &query) {
@@ -522,7 +530,7 @@ static vector<MediaFile> collect_media(Statement &query) {
 
 MediaFile MediaStorePrivate::lookup(const std::string &filename) const {
     Statement query(db, R"(
-SELECT filename, content_type, etag, title, date, artist, album, album_artist, genre, disc_number, track_number, duration, width, height, latitude, longitude, has_thumbnail, type
+SELECT filename, content_type, etag, title, date, artist, album, album_artist, genre, disc_number, track_number, duration, width, height, latitude, longitude, has_thumbnail, mtime, type
   FROM media
   WHERE filename = ?
 )");
@@ -535,7 +543,7 @@ SELECT filename, content_type, etag, title, date, artist, album, album_artist, g
 
 vector<MediaFile> MediaStorePrivate::query(const std::string &core_term, MediaType type, const Filter &filter) const {
     string qs(R"(
-SELECT filename, content_type, etag, title, date, artist, album, album_artist, genre, disc_number, track_number, duration, width, height, latitude, longitude, has_thumbnail, type
+SELECT filename, content_type, etag, title, date, artist, album, album_artist, genre, disc_number, track_number, duration, width, height, latitude, longitude, has_thumbnail, mtime, type
   FROM media
 )");
     if (!core_term.empty()) {
@@ -543,7 +551,7 @@ SELECT filename, content_type, etag, title, date, artist, album, album_artist, g
   JOIN (
     SELECT docid, rank(matchinfo(media_fts), 1.0, 0.5, 0.75) AS rank
       FROM media_fts WHERE media_fts MATCH ?
-    ) AS ranktable ON (media.rowid = ranktable.docid)
+    ) AS ranktable ON (media.id = ranktable.docid)
 )";
     }
     qs += " WHERE type = ?";
@@ -570,6 +578,12 @@ SELECT filename, content_type, etag, title, date, artist, album, album_artist, g
             qs += " DESC";
         }
         break;
+    case MediaOrder::Modified:
+        qs += " ORDER BY mtime";
+        if (filter.getReverse()) {
+            qs += " DESC";
+        }
+        break;
     }
     qs += " LIMIT ? OFFSET ?";
 
@@ -604,11 +618,11 @@ static vector<Album> collect_albums(Statement &query) {
 
 vector<Album> MediaStorePrivate::queryAlbums(const std::string &core_term, const Filter &filter) const {
     string qs(R"(
-SELECT album, album_artist, first(date) as date, first(genre) as genre, first(filename) as filename, first(has_thumbnail) as has_thumbnail FROM media
+SELECT album, album_artist, first(date) as date, first(genre) as genre, first(filename) as filename, first(has_thumbnail) as has_thumbnail, first(mtime) as mtime FROM media
 WHERE type = ? AND album <> ''
 )");
     if (!core_term.empty()) {
-        qs += " AND rowid IN (SELECT docid FROM media_fts WHERE media_fts MATCH ?)";
+        qs += " AND id IN (SELECT docid FROM media_fts WHERE media_fts MATCH ?)";
     }
     qs += " GROUP BY album, album_artist";
     switch (filter.getOrder()) {
@@ -623,6 +637,12 @@ WHERE type = ? AND album <> ''
         throw std::runtime_error("Can not query albums by rank");
     case MediaOrder::Date:
         throw std::runtime_error("Can not query albums by date");
+    case MediaOrder::Modified:
+        qs += " ORDER BY mtime";
+        if (filter.getReverse()) {
+            qs += " DESC";
+        }
+        break;
     }
     qs += " LIMIT ? OFFSET ?";
 
@@ -643,7 +663,7 @@ SELECT artist FROM media
 WHERE type = ? AND artist <> ''
 )");
     if (!q.empty()) {
-        qs += "AND rowid IN (SELECT docid FROM media_fts WHERE media_fts MATCH ?)";
+        qs += "AND id IN (SELECT docid FROM media_fts WHERE media_fts MATCH ?)";
     }
     qs += " GROUP BY artist";
     switch (filter.getOrder()) {
@@ -658,6 +678,8 @@ WHERE type = ? AND artist <> ''
         throw std::runtime_error("Can not query artists by rank");
     case MediaOrder::Date:
         throw std::runtime_error("Can not query artists by date");
+    case MediaOrder::Modified:
+        throw std::runtime_error("Can not query artists by modification date");
     }
     qs += " LIMIT ? OFFSET ?";
 
@@ -678,7 +700,7 @@ WHERE type = ? AND artist <> ''
 
 vector<MediaFile> MediaStorePrivate::getAlbumSongs(const Album& album) const {
     Statement query(db, R"(
-SELECT filename, content_type, etag, title, date, artist, album, album_artist, genre, disc_number, track_number, duration, width, height, latitude, longitude, type FROM media
+SELECT filename, content_type, etag, title, date, artist, album, album_artist, genre, disc_number, track_number, duration, width, height, latitude, longitude, has_thumbnail, mtime, type FROM media
 WHERE album = ? AND album_artist = ? AND type = ?
 ORDER BY disc_number, track_number
 )");
@@ -702,7 +724,7 @@ SELECT etag FROM media WHERE filename = ?
 
 std::vector<MediaFile> MediaStorePrivate::listSongs(const Filter &filter) const {
     std::string qs(R"(
-SELECT filename, content_type, etag, title, date, artist, album, album_artist, genre, disc_number, track_number, duration, width, height, latitude, longitude, has_thumbnail, type
+SELECT filename, content_type, etag, title, date, artist, album, album_artist, genre, disc_number, track_number, duration, width, height, latitude, longitude, has_thumbnail, mtime, type
   FROM media
   WHERE type = ?
 )");
@@ -857,6 +879,24 @@ SELECT genre FROM media
     return genres;
 }
 
+bool MediaStorePrivate::hasMedia(MediaType type) const {
+    if (type == AllMedia) {
+        Statement query(db, R"(
+SELECT id FROM media
+  LIMIT 1
+)");
+        return query.step();
+    } else {
+        Statement query(db, R"(
+SELECT id FROM media
+  WHERE type = ?
+  LIMIT 1
+)");
+        query.bind(1, (int)type);
+        return query.step();
+    }
+}
+
 void MediaStorePrivate::pruneDeleted() {
     std::map<std::string, bool> path_cache;
     vector<string> deleted;
@@ -991,6 +1031,10 @@ std::vector<std::string> MediaStore::listGenres(const Filter &filter) const {
     return p->listGenres(filter);
 }
 
+bool MediaStore::hasMedia(MediaType type) const {
+    std::lock_guard<std::mutex> lock(p->dbMutex);
+    return p->hasMedia(type);
+}
 
 size_t MediaStore::size() const {
     std::lock_guard<std::mutex> lock(p->dbMutex);
